/****************************************************************************

Copyright 2007 Virginia Polytechnic Institute and State University

This file is part of the OSSIE dsrc_display_samples.

OSSIE dsrc_display_samples is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

OSSIE dsrc_display_samples is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with OSSIE dsrc_display_samples; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

****************************************************************************/


#include <string>
#include <iostream>
#include "dsrc_display_samples.h"

dsrc_display_samples_i::dsrc_display_samples_i(const char *uuid, omni_condition *condition) :
	Resource_impl(uuid), component_running(condition)
{
	dataIn_0 = new standardInterfaces_i::realShort_p("in");

	//Create the thread for the writer's processing function
	processing_thread = new omni_thread(Run, (void *) this);

	//Start the thread containing the writer's processing function
	processing_thread->start();

}

dsrc_display_samples_i::~dsrc_display_samples_i(void)
{
	delete dataIn_0;
}

// Static function for omni thread
void dsrc_display_samples_i::Run( void * data )
{
	((dsrc_display_samples_i*)data)->ProcessData();
}

CORBA::Object_ptr dsrc_display_samples_i::getPort( const char* portName ) throw (
	CORBA::SystemException, CF::PortSupplier::UnknownPort)
{
	DEBUG(3, dsrc_display_samples, "getPort() invoked with " << portName)

	CORBA::Object_var p;

	p = dataIn_0->getPort(portName);

	if (!CORBA::is_nil(p))
		return p._retn();

	/*exception*/
	throw CF::PortSupplier::UnknownPort();
}

void dsrc_display_samples_i::start() throw (CORBA::SystemException,
	CF::Resource::StartError)
{
	DEBUG(3, dsrc_display_samples, "start() invoked")
}

void dsrc_display_samples_i::stop() throw (CORBA::SystemException, CF::Resource::StopError)
{
	DEBUG(3, dsrc_display_samples, "stop() invoked")
}

void dsrc_display_samples_i::releaseObject() throw (CORBA::SystemException,
	CF::LifeCycle::ReleaseError)
{
	DEBUG(3, dsrc_display_samples, "releaseObject() invoked")

	component_running->signal();
}

void dsrc_display_samples_i::initialize() throw (CF::LifeCycle::InitializeError,
	CORBA::SystemException)
{
	DEBUG(3, dsrc_display_samples, "initialize() invoked")
}

void dsrc_display_samples_i::configure(const CF::Properties& props)
throw (CORBA::SystemException,
	CF::PropertySet::InvalidConfiguration,
	CF::PropertySet::PartialConfiguration)
{
	DEBUG(3, dsrc_display_samples, "configure() invoked")

	std::cout << "props length : " << props.length() << std::endl;

	for (unsigned int i = 0; i <props.length(); i++)
	{
		std::cout << "Property id : " << props[i].id << std::endl;

	}
}

void dsrc_display_samples_i::ProcessData()
{
	DEBUG(3, dsrc_display_samples, "ProcessData() invoked")



	PortTypes::ShortSequence *I_in_0(NULL);
	CORBA::UShort I_in_0_length;

	while(1)
	{
		dataIn_0->getData(I_in_0);

		I_in_0_length = I_in_0->length();

		for (short int i = 0; i < I_in_0_length; i++)
		{
			std::cout << "Symbol " << i << ": " << (*I_in_0)[i] << std::endl;
		}

		dataIn_0->bufferEmptied();
	}
}
