/****************************************************************************

Copyright 2007 Virginia Polytechnic Institute and State University

This file is part of the OSSIE dsrc_pulse_shaper.

OSSIE dsrc_pulse_shaper is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

OSSIE dsrc_pulse_shaper is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with OSSIE dsrc_pulse_shaper; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

****************************************************************************/


#include <string>
#include <iostream>
#include "dsrc_pulse_shaper.h"

dsrc_pulse_shaper_i::dsrc_pulse_shaper_i(const char *uuid, omni_condition *condition) : 
    Resource_impl(uuid), component_running(condition) 
{
    dataIn_0 = new standardInterfaces_i::realChar_p("dataIn");
    dataOut_0 = new standardInterfaces_i::realShort_u("samplesOut");

    //Create the thread for the writer's processing function 
    processing_thread = new omni_thread(Run, (void *) this);

    //Start the thread containing the writer's processing function 
    processing_thread->start();

}

dsrc_pulse_shaper_i::~dsrc_pulse_shaper_i(void)
{   
    delete dataIn_0;
    delete dataOut_0;
}

// Static function for omni thread
void dsrc_pulse_shaper_i::Run( void * data )
{
    ((dsrc_pulse_shaper_i*)data)->ProcessData();
}

CORBA::Object_ptr dsrc_pulse_shaper_i::getPort( const char* portName ) throw (
    CORBA::SystemException, CF::PortSupplier::UnknownPort)
{
    DEBUG(3, dsrc_pulse_shaper, "getPort() invoked with " << portName)
    
    CORBA::Object_var p;

    p = dataIn_0->getPort(portName);

    if (!CORBA::is_nil(p))
        return p._retn();

    p = dataOut_0->getPort(portName);

    if (!CORBA::is_nil(p))
        return p._retn();

    /*exception*/
    throw CF::PortSupplier::UnknownPort();
}

void dsrc_pulse_shaper_i::start() throw (CORBA::SystemException, 
    CF::Resource::StartError)
{
    DEBUG(3, dsrc_pulse_shaper, "start() invoked")
}

void dsrc_pulse_shaper_i::stop() throw (CORBA::SystemException, CF::Resource::StopError) 
{  
    DEBUG(3, dsrc_pulse_shaper, "stop() invoked")
}

void dsrc_pulse_shaper_i::releaseObject() throw (CORBA::SystemException,
    CF::LifeCycle::ReleaseError)
{
    DEBUG(3, dsrc_pulse_shaper, "releaseObject() invoked")
    
    component_running->signal();
}

void dsrc_pulse_shaper_i::initialize() throw (CF::LifeCycle::InitializeError,
    CORBA::SystemException)
{
    DEBUG(3, dsrc_pulse_shaper, "initialize() invoked")
}

void dsrc_pulse_shaper_i::configure(const CF::Properties& props)
throw (CORBA::SystemException,
    CF::PropertySet::InvalidConfiguration,
    CF::PropertySet::PartialConfiguration)
{
    DEBUG(3, dsrc_pulse_shaper, "configure() invoked")
    
    std::cout << "props length : " << props.length() << std::endl;

    for (unsigned int i = 0; i <props.length(); i++)
    {
        std::cout << "Property id : " << props[i].id << std::endl;

        if (strcmp(props[i].id, "DCE:91b6c8c8-34bc-11de-93cb-000c29aafdf2") == 0)
        {
            CORBA::Short simple_temp;
            props[i].value >>= simple_temp;
            simple_0_value = simple_temp;
        }

        if (strcmp(props[i].id, "DCE:9c88f096-34bc-11de-93cb-000c29aafdf2") == 0)
        {
            CORBA::Short simple_temp;
            props[i].value >>= simple_temp;
            simple_1_value = simple_temp;
        }

        if (strcmp(props[i].id, "DCE:9abd2ccc-34c7-11de-93cb-000c29aafdf2") == 0)
        {
            CORBA::Short simple_temp;
            props[i].value >>= simple_temp;
            simple_2_value = simple_temp;
        }

        if (strcmp(props[i].id, "DCE:a44ab84a-34c7-11de-93cb-000c29aafdf2") == 0)
        {
            CORBA::Short simple_temp;
            props[i].value >>= simple_temp;
            simple_3_value = simple_temp;
        }

    }
}

void dsrc_pulse_shaper_i::ProcessData()
{
    DEBUG(3, dsrc_pulse_shaper, "ProcessData() invoked")

    PortTypes::ShortSequence I_out_0;


    PortTypes::CharSequence *I_in_0(NULL);
    CORBA::UShort I_in_0_length;

    while(1)
    {
        dataIn_0->getData(I_in_0);

        I_in_0_length = I_in_0->length();

        I_out_0.length(); //must define length of output

        /*insert code here to do work*/






        dataIn_0->bufferEmptied();
        dataOut_0->pushPacket(I_out_0);
    }
}


