/************************************************************************

Copyright 2009 Einar Thorsrud
Copyright 2007 Virginia Polytechnic Institute and State University

This file is part of the OSSIE dsrc_pulse_shaper.

OSSIE dsrc_pulse_shaper is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License as published
by the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

OSSIE dsrc_pulse_shaper is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with OSSIE dsrc_pulse_shaper; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

************************************************************************/

#include <string>
#include <iostream>
#include "dsrc_pulse_shaper.h"

dsrc_pulse_shaper_i::dsrc_pulse_shaper_i(const char *uuid, omni_condition *condition) :
	Resource_impl(uuid), component_running(condition)
{
	dataIn_0 = new standardInterfaces_i::realChar_p("dataIn");
	dataOut_0 = new standardInterfaces_i::realShort_u("samplesOut");

	//Create the thread for the writer's processing function
	processing_thread = new omni_thread(Run, (void *) this);

	//Start the thread containing the writer's processing function
	processing_thread->start();

}

dsrc_pulse_shaper_i::~dsrc_pulse_shaper_i(void)
{
	delete dataIn_0;
	delete dataOut_0;
}

// Static function for omni thread
void dsrc_pulse_shaper_i::Run( void * data )
{
	((dsrc_pulse_shaper_i*)data)->ProcessData();
}

CORBA::Object_ptr dsrc_pulse_shaper_i::getPort( const char* portName ) throw (
	CORBA::SystemException, CF::PortSupplier::UnknownPort)
{
	DEBUG(3, dsrc_pulse_shaper, "getPort() invoked with " << portName)

	CORBA::Object_var p;

	p = dataIn_0->getPort(portName);

	if (!CORBA::is_nil(p))
		return p._retn();

	p = dataOut_0->getPort(portName);

	if (!CORBA::is_nil(p))
		return p._retn();

	/*exception*/
	throw CF::PortSupplier::UnknownPort();
}

void dsrc_pulse_shaper_i::start() throw (CORBA::SystemException,
	CF::Resource::StartError)
{
	DEBUG(3, dsrc_pulse_shaper, "start() invoked")
}

void dsrc_pulse_shaper_i::stop() throw (CORBA::SystemException, CF::Resource::StopError)
{
	DEBUG(3, dsrc_pulse_shaper, "stop() invoked")
}

void dsrc_pulse_shaper_i::releaseObject() throw (CORBA::SystemException,
	CF::LifeCycle::ReleaseError)
{
	DEBUG(3, dsrc_pulse_shaper, "releaseObject() invoked")

	component_running->signal();
}

void dsrc_pulse_shaper_i::initialize() throw (CF::LifeCycle::InitializeError,
	CORBA::SystemException)
{
	DEBUG(3, dsrc_pulse_shaper, "initialize() invoked")
}

void dsrc_pulse_shaper_i::configure(const CF::Properties& props)
throw (CORBA::SystemException,
	CF::PropertySet::InvalidConfiguration,
	CF::PropertySet::PartialConfiguration)
{
	DEBUG(3, dsrc_pulse_shaper, "configure() invoked")

	//std::cout << "props length : " << props.length() << std::endl;

	for (unsigned int i = 0; i <props.length(); i++)
	{
		//std::cout << "Property id : " << props[i].id << std::endl;

		if (strcmp(props[i].id, "DCE:91b6c8c8-34bc-11de-93cb-000c29aafdf2") == 0)
		{
			CORBA::Short simple_temp;
			props[i].value >>= simple_temp;
			simple_0_value = simple_temp;
		}

		if (strcmp(props[i].id, "DCE:9c88f096-34bc-11de-93cb-000c29aafdf2") == 0)
		{
			CORBA::Short simple_temp;
			props[i].value >>= simple_temp;
			simple_1_value = simple_temp;
		}

		if (strcmp(props[i].id, "DCE:9abd2ccc-34c7-11de-93cb-000c29aafdf2") == 0)
		{
			CORBA::Short simple_temp;
			props[i].value >>= simple_temp;
			simple_2_value = simple_temp;
		}

		if (strcmp(props[i].id, "DCE:a44ab84a-34c7-11de-93cb-000c29aafdf2") == 0)
		{
			CORBA::Short simple_temp;
			props[i].value >>= simple_temp;
			simple_3_value = simple_temp;
		}

	}
}

void dsrc_pulse_shaper_i::ProcessData()
{
	DEBUG(3, dsrc_pulse_shaper, "ProcessData() invoked")

	PortTypes::ShortSequence I_out_0;


	PortTypes::CharSequence *I_in_0(NULL);
	CORBA::UShort I_in_0_length;


	const int HALF_SIN_LENGTH = 256;

	short half_sin[HALF_SIN_LENGTH] = {
		   0,   25,   50,   75,  100,  125,  150,  175,
		 200,  225,  250,  275,  300,  325,  349,  374,
		 399,  423,  448,  472,  497,  521,  545,  569,
		 593,  617,  641,  665,  688,  712,  735,  759,
		 782,  805,  828,  851,  874,  896,  919,  941,
		 963,  985, 1007, 1029, 1050, 1072, 1093, 1114,
		1135, 1155, 1176, 1196, 1216, 1236, 1256, 1276,
		1295, 1315, 1334, 1352, 1371, 1389, 1408, 1426,
		1443, 1461, 1478, 1495, 1512, 1529, 1545, 1561,
		1577, 1593, 1608, 1624, 1639, 1653, 1668, 1682,
		1696, 1710, 1723, 1736, 1749, 1762, 1774, 1786,
		1798, 1810, 1821, 1832, 1843, 1853, 1863, 1873,
		1883, 1892, 1901, 1910, 1919, 1927, 1935, 1942,
		1949, 1956, 1963, 1970, 1976, 1981, 1987, 1992,
		1997, 2002, 2006, 2010, 2014, 2017, 2020, 2023,
		2025, 2027, 2029, 2031, 2032, 2033, 2034, 2034,
		2034, 2034, 2033, 2032, 2031, 2029, 2027, 2025,
		2023, 2020, 2017, 2014, 2010, 2006, 2002, 1997,
		1992, 1987, 1981, 1976, 1970, 1963, 1956, 1949,
		1942, 1935, 1927, 1919, 1910, 1901, 1892, 1883,
		1873, 1863, 1853, 1843, 1832, 1821, 1810, 1798,
		1786, 1774, 1762, 1749, 1736, 1723, 1710, 1696,
		1682, 1668, 1653, 1639, 1624, 1608, 1593, 1577,
		1561, 1545, 1529, 1512, 1495, 1478, 1461, 1443,
		1426, 1408, 1389, 1371, 1352, 1334, 1315, 1295,
		1276, 1256, 1236, 1216, 1196, 1176, 1155, 1135,
		1114, 1093, 1072, 1050, 1029, 1007,  985,  963,
		 941,  919,  896,  874,  851,  828,  805,  782,
		 759,  735,  712,  688,  665,  641,  617,  593,
		 569,  545,  521,  497,  472,  448,  423,  399,
		 374,  349,  325,  300,  275,  250,  225,  200,
		 175,  150,  125,  100,   75,   50,   25,    0
	};

	short half_sin2[HALF_SIN_LENGTH] = {
		  25,   50,   75,  100,  126,  151,  176,  201,
		 226,  251,  276,  301,  325,  350,  375,  400,
		 424,  449,  473,  498,  522,  546,  570,  595,
		 619,  642,  666,  690,  714,  737,  760,  784,
		 807,  830,  853,  876,  898,  921,  943,  965,
		 988, 1009, 1030, 1052, 1073, 1095, 1116, 1137,
		1158, 1178, 1199, 1219, 1239, 1259, 1279, 1298,
		1318, 1337, 1356, 1374, 1393, 1411, 1429, 1447,
		1465, 1482, 1499, 1516, 1533, 1550, 1566, 1582,
		1598, 1614, 1629, 1644, 1659, 1673, 1688, 1702,
		1716, 1729, 1743, 1756, 1768, 1781, 1793, 1805,
		1817, 1828, 1839, 1850, 1861, 1871, 1881, 1891,
		1901, 1910, 1919, 1927, 1936, 1944, 1951, 1959,
		1966, 1973, 1979, 1986, 1992, 1997, 2003, 2008,
		2012, 2017, 2021, 2025, 2028, 2032, 2035, 2037,
		2039, 2041, 2043, 2045, 2046, 2046, 2047, 2047,
		2047, 2046, 2046, 2045, 2043, 2041, 2039, 2037,
		2035, 2032, 2028, 2025, 2021, 2017, 2012, 2008,
		2003, 1997, 1992, 1986, 1979, 1973, 1966, 1959,
		1951, 1944, 1936, 1927, 1919, 1910, 1901, 1891,
		1881, 1871, 1861, 1850, 1843, 1834, 1826, 1818,
		1810, 1802, 1793, 1784, 1776, 1769, 1761, 1753,
		1745, 1738, 1731, 1726, 1720, 1714, 1709, 1704,
		1698, 1693, 1688, 1682, 1677, 1672, 1667, 1662,
		1657, 1652, 1648, 1643, 1638, 1633, 1629, 1624,
		1620, 1616, 1611, 1607, 1603, 1599, 1595, 1591,
		1587, 1583, 1580, 1576, 1572, 1569, 1565, 1562,
		1559, 1555, 1552, 1549, 1546, 1543, 1540, 1538,
		1535, 1532, 1530, 1527, 1525, 1523, 1520, 1518,
		1516, 1514, 1512, 1511, 1509, 1507, 1506, 1504,
		1503, 1502, 1500, 1499, 1498, 1497, 1496, 1495,
		1495, 1494, 1494, 1493, 1493, 1492, 1492, 1492
	};

	short max, min, sign, interpol_fac;


	while(1)
	{

		max = (short) simple_0_value;
		min = (short) simple_1_value;
		sign = (short) simple_2_value; // Must be read only once
		interpol_fac = (short) simple_3_value;


		//std::cout << "max: " << max << std::endl;
		//std::cout << "min: " << min << std::endl;
		//std::cout << "sign: " << sign << std::endl;
		//std::cout << "interpol_fac: " << interpol_fac << std::endl;

		dataIn_0->getData(I_in_0);
		I_in_0_length = I_in_0->length();
		I_out_0.length(I_in_0_length * interpol_fac); //must define length of output

		//int step_lenght = (256 * 2) / (int)interpol_fac;
		int step_lenght = (256 * 2) / 8; // Hardcoded for the moment - not good!

		const short TABLE_LEVEL = 2047; // maximum absolute value of the signal
		const float scaling = (float)(max - min) / (float)(TABLE_LEVEL * 2);
		const float DC_offset = min + TABLE_LEVEL * scaling + 0.5; // added 0.5 for correct round off when casting to short

		// Har en misstanke om at sign ødelegger ved å bli lest for hver pakke
		//  - mulig det er bedre å droppe den slik at det alltid anntas at
		// den forrige var enten det ene eller det andre.

		for (int i = 0; i < I_in_0_length; i++)
		{
			if ((*I_in_0)[i] == 0)
			{
				for (int j = 0; j <  interpol_fac/2; j++)
				{
					(I_out_0)[interpol_fac * i + j] = (short) half_sin[step_lenght * j] * sign * scaling + DC_offset;
					if ((I_out_0)[interpol_fac * i + j] < 0)
						(I_out_0)[interpol_fac * i + j]--;
				}
				for (int j = interpol_fac/2; j <  interpol_fac; j++)
				{
					(I_out_0)[interpol_fac * i + j] = (short) -half_sin[step_lenght * j - HALF_SIN_LENGTH] * sign * scaling + DC_offset; // - HALF_SIN_LENGTH to read from pos. zero
					if ((I_out_0)[interpol_fac * i + j] < 0)
						(I_out_0)[interpol_fac * i + j]--;
				}
			}

			else if ((*I_in_0)[i] == 1)
			{
				for (int j = 0; j <  interpol_fac/2; j++)
				{
					(I_out_0)[interpol_fac * i + j] = (short) half_sin2[step_lenght * j] * sign * scaling + DC_offset;
					if ((I_out_0)[interpol_fac * i + j] < 0)
						(I_out_0)[interpol_fac * i + j]--;
				}
				for (int j = interpol_fac/2; j <  interpol_fac; j++)
				{
					(I_out_0)[interpol_fac * i + j] = (short) half_sin2[HALF_SIN_LENGTH * 2 - step_lenght * j - 1] * sign * scaling + DC_offset; // - HALF_SIN_LENGTH to read from pos. zero
					if ((I_out_0)[interpol_fac * i + j] < 0)
						(I_out_0)[interpol_fac * i + j]--;
				}
			sign = -sign;
			}
		}


		dataIn_0->bufferEmptied();
		dataOut_0->pushPacket(I_out_0);
	}
}
