/* -*- c++ -*- */
/*
 * Copyright 2009 Einar Thorsrud.
 *
 * This file is part of the GNU Radio DSRC module.
 *
 * The GNU Radio DSRC module is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2, or (at your option) any later version.
 *
 * The GNU Radio DSRC module is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GNU Radio DSRC module; see the file COPYING.  If
 * not, write to the Free Software Foundation, Inc., 51 Franklin
 * Street, Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <dsrc_fm0_to_nrz_bb.h>
#include <gr_io_signature.h>
#include <stdexcept>

// bør kanskje være en parameter etterhver, gjør "noutput_items" korrekt
static const int INPUT_PER_OUTPUT = 2;


/*
 * Create a new instance of dsrc_fm0_modulator_bc and return
 * a boost shared_ptr.  This is effectively the public constructor.
 */
dsrc_fm0_to_nrz_bb_sptr
dsrc_make_fm0_to_nrz_bb ()
{
  return dsrc_fm0_to_nrz_bb_sptr (new dsrc_fm0_to_nrz_bb ());
}


/*
 * Specify constraints on number of input and output streams.
 * This info is used to construct the input and output signatures
 * (2nd & 3rd args to gr_block's constructor).  The input and
 * output signatures are used by the runtime system to
 * check that a valid number and type of inputs and outputs
 * are connected to this block.  In this case, we accept
 * only 1 input and 1 output.
 */
static const int MIN_IN = 1;  // mininum number of input streams
static const int MAX_IN = 1;  // maximum number of input streams
static const int MIN_OUT = 1; // minimum number of output streams
static const int MAX_OUT = 1; // maximum number of output streams



/*
 * The private constructor
 */
dsrc_fm0_to_nrz_bb::dsrc_fm0_to_nrz_bb ()
  : gr_sync_decimator ("fm0_modulator_bc",
                         gr_make_io_signature (MIN_IN, MAX_IN, sizeof (unsigned char)), 
                         gr_make_io_signature (MIN_OUT, MAX_OUT, sizeof (unsigned char)),
                         INPUT_PER_OUTPUT)

{
  // nothing else required in this block (dsrc_fm0_modulator_bc)
}


// Kunne hatt en virtuell destruktor her!!!


int
dsrc_fm0_to_nrz_bb::work (int                       noutput_items,
                         gr_vector_const_void_star &input_items,
                         gr_vector_void_star       &output_items)
{

  const unsigned char *in = (const unsigned char *) input_items[0];
  unsigned char *out = (unsigned char *) output_items[0];

  for (int i = 0; i < noutput_items * INPUT_PER_OUTPUT; i++){
    if (in[i*2] == in[i*2+1]) {
        out[i] = true;
    } else { // in[i*2] != in[i*2+1]
        out[i] = false;
    }   
  }

  // Tell runtime system how many output items we produced.
  return noutput_items; // Two samples per symbol (for now)
}
