/* -*- c++ -*- */
/*
 * Copyright 2009 Einar Thorsrud.
 *
 * This file is part of the GNU Radio DSRC module.
 *
 * The GNU Radio DSRC module is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2, or (at your option) any later version.
 *
 * The GNU Radio DSRC module is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GNU Radio DSRC module; see the file COPYING.  If
 * not, write to the Free Software Foundation, Inc., 51 Franklin
 * Street, Boston, MA 02110-1301, USA.
 */

#ifndef INCLUDED_PULS_SHAPER_BS_H
#define INCLUDED_PULS_SHAPER_BS_H

// This block is based on the gr_sync_interpolator-block
#include <gr_sync_interpolator.h>

class dsrc_pulse_shaper_bs;
typedef boost::shared_ptr<dsrc_pulse_shaper_bs> dsrc_pulse_shaper_bs_sptr;

dsrc_pulse_shaper_bs_sptr dsrc_make_pulse_shaper_bs (short v_max, short v_min, int preload, int interpolation);

/*!
 * \brief Modulate FM0-signal on bit at a time.
 * \ingroup dsrc
 *
 * input: stream of byte
 * output: stream of byte
 *
 */
class dsrc_pulse_shaper_bs : public gr_sync_interpolator
{
  friend dsrc_pulse_shaper_bs_sptr dsrc_make_pulse_shaper_bs (short v_max, short v_min, int preload, int interpolation);
  dsrc_pulse_shaper_bs (short v_max, short v_min, int preload, int interpolation);


  short max;
  short min;
  int sign; // used as 1 or -1
  int interpol_fac;
  
 public:
  int work (int                       noutput_items,
            gr_vector_const_void_star &input_items,
            gr_vector_void_star       &output_items);
};

#endif
