#!/usr/bin/env python
#
# Copyright 2009 Einar Thorsrud.
#
# This file is part of the GNU Radio DSRC module.
#
# The GNU Radio DSRC module is free software; you can redistribute
# it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation; either
# version 2, or (at your option) any later version.
#
# The GNU Radio DSRC module is distributed in the hope that it will
# be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the GNU Radio DSRC module; see the file COPYING.  If
# not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Boston, MA 02110-1301, USA.
# 

from gnuradio import gr, gr_unittest
import dsrc

class qa_dsrc (gr_unittest.TestCase):

	def setUp (self):
		self.tb = gr.top_block ()

	def tearDown (self):
		self.tb = None

	def test_001_fm0_modulator_bc (self):
		src_data = (1, 1, 0, 0, 1, 0)
		expected_result = (0, 0, 1, 1, 0, 1, 0, 1, 0, 0, 1, 0)		
		src = gr.vector_source_b (src_data)
		data_to_fm0 = dsrc.nrz_to_fm0_bb (1) # Test with initial state v_max
		dst = gr.vector_sink_b ()
		self.tb.connect (src, data_to_fm0)
		self.tb.connect (data_to_fm0, dst)
		self.tb.run ()
		result_data = dst.data ()
		self.assertEqual (expected_result, result_data)

	def test_002_fm0_modulator_bc (self):
		src_data = (1, 1, 0, 0, 1, 0)
		expected_result = (1, 1, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1)
		src = gr.vector_source_b (src_data)
		data_to_fm0 = dsrc.nrz_to_fm0_bb (0) # Test with initial state v_min
		dst = gr.vector_sink_b ()
		self.tb.connect (src, data_to_fm0)
		self.tb.connect (data_to_fm0, dst)
		self.tb.run ()
		result_data = dst.data ()
		self.assertEqual (expected_result, result_data)

if __name__ == '__main__':
	gr_unittest.main ()
